/* This is a simple test client program that does a CSTA MakeCall       */
/* SOAP message exchange with the sample server application             */
/* (server.cpp).  The request is sent to the server using HTTP / SOAP / */
/* XML.  The client then waits for a response from the server, displays */
/* the result, and exits..                                              */

#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/rtxSOAP.h"
#include "rtxsrc/rtxSocket.h"
#include "rtxsrc/rtxStreamMemory.h"
#include "rtxsrc/rtxStreamSocket.h"
#include "rtxsrc/OSRTMemoryInputStream.h"
#include "rtxmlsrc/OSXMLDecodeBuffer.h"
#include "rtxmlsrc/OSXMLEncodeBuffer.h"
#include "csta.h"

int main (int argc, char** argv)
{
   OSSOAPCONN   soapConn;
   OSOCTET*     ptmpbuf = 0;
   int		i, stat;
   const char*  url = "http://127.0.0.1:9125";
   OSBOOL       verbose = FALSE, trace = TRUE;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-url")) url = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: client [-v] [-url <url>] [-notrace]\n");
            printf ("   -v    verbose mode: print trace info\n");
            printf ("   -url  connect to <url>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   // Create buffer and control objects

   OSXMLEncodeBuffer buffer;
   MakeCall_ELEM data;
   MakeCall_CC pdu (buffer, data);

   buffer.setDiag (verbose);

   // One-time socket package initialization (required for Windows only)

   stat = rtxSocketsInit();
   if (0 != stat) {
      printf ("Socket initialization failed.\n");
      rtxErrSetData (buffer.getCtxtPtr(), stat, __FILE__, __LINE__);
      buffer.printErrorInfo();
      return stat;
   }

   // Populate structure of generated type

   MakeCall_ELEM* pvalue = &data;
   pvalue->callingDevice.value.set_number (new Number("22343"));
   pvalue->calledDirectoryNumber.value.set_number (new Number("18005551212"));
   pvalue->m.accountCodePresent = 1;
   {
      static const OSOCTET bindata[] = { 0x00, 0x01 };
      pvalue->accountCode.setValue (sizeof(bindata), bindata);
   }
   pvalue->m.authCodePresent = 1;
   {
      static const OSOCTET bindata[] = { 0x00, 0x01 };
      pvalue->authCode.setValue (sizeof(bindata), bindata);
   }
   pvalue->m.autoOriginatePresent = 1;
   pvalue->autoOriginate = AutoOriginate::prompt;
   pvalue->m.correlatorDataPresent = 1;
   pvalue->correlatorData.t = 1;
   pvalue->correlatorData.u.string = new OSDynOctStrClass();
   {
      static const OSOCTET bindata[] = { 0x00, 0x01 };
      pvalue->correlatorData.u.string->setValue (2, bindata);
   }
   pvalue->m.userDataPresent = 1;
   pvalue->userData.t = 1;
   pvalue->userData.u.string = new OSDynOctStrClass();
   {
      static const OSOCTET bindata[] = { 0x00, 0x01 };
      pvalue->userData.u.string->setValue (2, bindata);
   }
   pvalue->m.callCharacteristicsPresent = 1;
   pvalue->callCharacteristics.m.priorityCallPresent = 1;
   pvalue->callCharacteristics.priorityCall = TRUE;
   pvalue->m.mediaCallCharacteristicsPresent = 1;
   pvalue->mediaCallCharacteristics.mediaClass.m.voicePresent = 1;
   pvalue->mediaCallCharacteristics.mediaClass.voice = TRUE;
   pvalue->m.callingConnectionInfoPresent = 1;
   pvalue->callingConnectionInfo.m.flowDirectionPresent = 1;
   pvalue->callingConnectionInfo.flowDirection
       = ConnectionInformation_flowDirection::transmitAndReceive;
   pvalue->callingConnectionInfo.m.numberOfChannelsPresent = 1;
   pvalue->callingConnectionInfo.numberOfChannels = 2;
   pvalue->m.extensionsPresent = 1;
   pvalue->extensions.m.securityPresent = 1;
   pvalue->extensions.security.m.messageSequenceNumberPresent = 1;
   pvalue->extensions.security.messageSequenceNumber = 123;
   pvalue->extensions.security.m.securityInfoPresent = 1;
   pvalue->extensions.security.securityInfo.t = 1;
   pvalue->extensions.security.securityInfo.u.string = new OSDynOctStrClass();
   {
      static const OSOCTET bindata[] = { 0x00, 0x01 };
      pvalue->extensions.security.securityInfo.u.string->setValue 
         (2, bindata);
   }
   pvalue->extensions.m.privateDataPresent = 1;
   pvalue->extensions.privateData.t = 1;
   pvalue->extensions.privateData.u.string = new OSDynOctStrClass();
   {
      static const OSOCTET bindata[] = { 0x00, 0x01 };
      pvalue->extensions.privateData.u.string->setValue (2, bindata);
   }

   // Add XML document and SOAP headers to encode buffer

   buffer.addXMLHeader ();

   stat = rtXmlEncStartSoapElems (buffer.getCtxtPtr(), OSSOAPBODY);

   // Encode SOAP body content

   if (0 == stat) {
      stat = pdu.encode();
   }

   // Close SOAP body and envelope elements

   if (0 == stat) {
      stat = rtXmlEncEndSoapElems (buffer.getCtxtPtr(), OSSOAPBODY);
   }

   rtXmlEncEndDocument (buffer.getCtxtPtr());

   if (0 == stat) {
      if (trace) {
         printf ("encoded XML message:\n");
         printf ((const char*)buffer.getMsgPtr());
         printf ("\n");
      }
   }
   else {
      printf ("Encoding failed\n");
      buffer.printErrorInfo();
      return stat;
   }

   /* Connect to server */

   rtxSoapInitConn (&soapConn, buffer.getCtxtPtr(), OSSOAPV1, url);

   stat = rtxSoapConnect (&soapConn);
   if (stat != 0) {
      buffer.printErrorInfo();
      return stat;
   }
   if (trace) printf ("Connection to web service opened\n");

   /* Send request */

   soapConn.soapAction = "MakeCall";

   stat = rtxSoapSendHttp (&soapConn, buffer.getMsgPtr());
   if (stat != 0) {
      buffer.printErrorInfo();
      return stat;
   }
   if (trace) printf ("HTTP request sent\n");

   /* Wait for response */

   stat = rtxSoapRecvHttp (&soapConn);
   if (stat != 0) {
      buffer.printErrorInfo();
      rtxStreamClose (buffer.getCtxtPtr());
      return stat;
   }
   if (trace) printf ("HTTP response received\n");

   /* Read HTTP response into memory */

   stat = rtxSoapRecvHttpContent (&soapConn, &ptmpbuf);
   if (stat != 0) {
      buffer.printErrorInfo();
      return stat;
   }

   /* Close connection */

   rtxStreamClose (buffer.getCtxtPtr());

   /* Decode SOAP XML message */

   OSRTMemoryInputStream in (ptmpbuf, soapConn.contentLength);
   OSXMLDecodeBuffer decodeBuffer (in);

   if (decodeBuffer.getStatus ()) {
      decodeBuffer.printErrorInfo();
      return decodeBuffer.getStatus ();
   }
   OSCTXT* pDecCtxt = decodeBuffer.getCtxtPtr();
   OSXMLElemDescr soapElem;

   // First 2 elements should be SOAP envelope and body

   for (int j = 0; j < 2; j++) {
      stat = rtXmlpGetNextElem (pDecCtxt, &soapElem, -1);
      if (0 == stat) {
         printf ("soapElem = '%.*s'\n", 
                 soapElem.localName.length, soapElem.localName.value);
      }
      else {
         printf ("rtXmlpGetNextElem failed\n");
         decodeBuffer.printErrorInfo();
         return stat;
      }
   }

   // Decode SOAP body content

   MakeCallResponse_ELEM value;
   MakeCallResponse_CC respPDU (decodeBuffer, value);

   decodeBuffer.setDiag (verbose);

   // Decode

   stat = respPDU.decode();

   if (0 == stat) {
      if (trace) {
         printf ("decoded XML message:\n");
         respPDU.print ("MakeCallResponse");
         printf ("\n");
      }
   }
   else {
      printf ("Decoding failed\n");
      decodeBuffer.printErrorInfo();
      return stat;
   }

   return stat;
}
